package com.example.takeawaysystemserver.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.takeawaysystemserver.constant.CommonStatusConstant;
import com.example.takeawaysystemserver.entity.*;
import com.example.takeawaysystemserver.mapper.OrderMapper;
import com.example.takeawaysystemserver.model.vo.DishVO;
import com.example.takeawaysystemserver.model.vo.OrderVO;
import com.example.takeawaysystemserver.service.*;
import com.example.takeawaysystemserver.util.R;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
* @author qiushengyu
* @description 针对表【order】的数据库操作Service实现
* @createDate 2024-07-11 17:13:46
*/
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, OrderList>
    implements OrderService{
    @Resource
    private UserServiceImpl userServiceImpl;

    @Resource
    private ShoppingCartService shoppingCartService;

    @Resource
    private UserService userService;

    @Resource
    private AddressService addressService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private DishService dishService;

    @Resource
    private ShopService shopService;

    @Override
    public R<String> addOrders(String token, OrderList order) {
        Integer id = Integer.parseInt(userServiceImpl.getUserId(token));
        LambdaQueryWrapper<ShoppingCart> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ShoppingCart::getUserId,id);
        List<ShoppingCart> shoppingCarts = shoppingCartService.list(wrapper);

        if (shoppingCarts == null || shoppingCarts.isEmpty()){
            return R.fail(CommonStatusConstant.SHOPPINGCART_ERROR);
        }

        User user = userService.getById(id);
        Integer addressId = order.getAddressId();
        String address = user.getAddress();
        if (address == null){
            return R.fail(CommonStatusConstant.ADDRESS_ERROR);
        }

        long orderId = IdWorker.getId();

        AtomicInteger amount = new AtomicInteger(0);

        List<OrderItem> orderItems = shoppingCarts.stream().map((item) -> {
            OrderItem orderItem = new OrderItem();
            orderItem.setOrderId(orderId);
            orderItem.setCount(item.getNumber());
            orderItem.setDishId(item.getDishId());
            orderItem.setPrice(item.getAmount());
            return orderItem;
        }).toList();

        order.setId(orderId);
        order.setUserId(id);
        order.setAddress(address);
        boolean save = this.save(order);
        if (!save){
            return R.fail(CommonStatusConstant.INSERT_ERROR);
        }
        boolean b = orderItemService.saveBatch(orderItems);
        if (!b){
            return R.fail(CommonStatusConstant.INSERT_ERROR);
        }
        shoppingCartService.remove(wrapper);

        return R.ok("添加成功");
    }

    @Override
    public R<List<OrderVO>> getOrders(String token) {
        Integer id = Integer.parseInt(userServiceImpl.getUserId(token));
        LambdaQueryWrapper<OrderList> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(OrderList::getUserId,id);
        List<OrderList> orderLists = list(wrapper);

        List<OrderVO> orders = orderLists.stream().map((item) -> {
            OrderVO orderVO = new OrderVO();
            LambdaQueryWrapper<OrderItem> wrapper1 = new LambdaQueryWrapper<>();
            wrapper1.eq(OrderItem::getOrderId, item.getId());
            List<OrderItem> orderItems = orderItemService.list(wrapper1);
            List<DishVO> dishVos = new ArrayList<>();
            int amount = 0;
            for (OrderItem orderItem : orderItems) {
                LambdaQueryWrapper<Dish> dishes = new LambdaQueryWrapper<>();
                dishes.eq(Dish::getId, orderItem.getDishId());
                String name = dishService.getOne(dishes).getName();
                int i = orderItem.getPrice() * orderItem.getCount();
                DishVO dishVO = new DishVO(orderItem.getDishId(),name,orderItem.getPrice(),orderItem.getCount(),i);
                dishVos.add(dishVO);
                amount += i;
            }
            String shopName = shopService.getById(item.getShopId()).getName();
            orderVO.setShopName(shopName);
            orderVO.setShopId(item.getShopId());
            orderVO.setOrderId(item.getId());
            orderVO.setDishes(dishVos);
            orderVO.setPrice(amount);
            return orderVO;
        }).toList();

        return R.ok(orders);
    }
}




