package com.example.takeawaysystemserver.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.takeawaysystemserver.entity.Shop;
import com.example.takeawaysystemserver.entity.ShoppingCart;
import com.example.takeawaysystemserver.service.ShoppingCartService;
import com.example.takeawaysystemserver.mapper.ShoppingCartMapper;
import com.example.takeawaysystemserver.service.UserService;
import com.example.takeawaysystemserver.util.R;
import com.example.takeawaysystemserver.util.TimeUtil;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.List;

/**
* @author qiushengyu
* @description 针对表【shopping_cart】的数据库操作Service实现
* @createDate 2024-07-12 16:13:47
*/
@Service
public class ShoppingCartServiceImpl extends ServiceImpl<ShoppingCartMapper, ShoppingCart>
    implements ShoppingCartService{

    @Resource
    private UserServiceImpl userServiceImpl;


    @Override
    public R<String> addShoppingCart(String token, ShoppingCart shoppingCart) {
        Integer id = Integer.parseInt(userServiceImpl.getUserId(token));
        shoppingCart.setUserId(id);
        LambdaQueryWrapper<ShoppingCart> wrapper = new LambdaQueryWrapper<ShoppingCart>();
        wrapper.eq(shoppingCart.getDishId() != null, ShoppingCart::getDishId, shoppingCart.getDishId());
        wrapper.eq(shoppingCart.getShopId() != null, ShoppingCart::getShopId, shoppingCart.getShopId());

        ShoppingCart one = getOne(wrapper);
        //根据商家ID、菜品ID查询数据库是否已有
        if (one != null){
            //有就+1
            shoppingCart.setId(one.getId());
            shoppingCart.setNumber(one.getNumber() + 1);
            updateById(shoppingCart);
        }else {
            //没有就添加购物车
            Integer createTime = TimeUtil.getCurTimestamp();
            shoppingCart.setCreateTime(createTime);
            save(shoppingCart);
        }
        return R.ok("添加成功");
    }

    @Override
    public R<List<ShoppingCart>> getShoppingCarts(String token) {
        Integer id = Integer.parseInt(userServiceImpl.getUserId(token));
        LambdaQueryWrapper<ShoppingCart> wrapper = new LambdaQueryWrapper<ShoppingCart>();
        wrapper.orderByDesc(ShoppingCart::getCreateTime);
        wrapper.eq(ShoppingCart::getUserId, id);
        return R.ok(list(wrapper));
    }

    @Override
    public R<String> deleteShoppingCart(String token, ShoppingCart shoppingCart) {
        Integer id = Integer.parseInt(userServiceImpl.getUserId(token));
        shoppingCart.setUserId(id);
        LambdaQueryWrapper<ShoppingCart> wrapper = new LambdaQueryWrapper<ShoppingCart>();
        wrapper.eq(shoppingCart.getDishId() != null, ShoppingCart::getDishId, shoppingCart.getDishId());
        wrapper.eq(shoppingCart.getShopId() != null, ShoppingCart::getShopId, shoppingCart.getShopId());
        ShoppingCart one = getOne(wrapper);

        if (one.getNumber() == 1){
            removeById(one.getId());
        }else {
            shoppingCart.setId(one.getId());
            shoppingCart.setNumber(one.getNumber() - 1);
            updateById(shoppingCart);
        }
        return R.ok("删除成功");
    }
}




